function is_ip4 (addr,	x) {

	x=split(addr,parts,".")
	if (x != 4) { return "Nope!" }
	for (i=1; i<=x; i++) {
		if ( parts[i] < 0 || parts[i] >255 ) { return "Nope!" }
	}
	return "IP4"
}


#
# Return decimal value of an IPv4 addr
#
function ip2dec (addr) {
	num=0
        x=split(addr,parts,".")
        for (i=1; i<=4; i++) {
                pot=256^(4-i)
		num=num+(pot*parts[i])
        }
	return num
}

#
# Return binary representation of a decimal number
# With 'len' specified the bin-string is padded with
# zeroes on the left
#
function num2bin (num,len) {
	bitstr=""
	while (num >0 ) {
		bitstr=num%2 bitstr
		num=rshift(num,1)
		len--
	}

	while (len-- > 0) {bitstr="0" bitstr}

	return bitstr
}

function num2ip (number) {

	mask=0xFF000000
	part[1]=rshift(and(number, mask), 24) 
	mask=rshift(mask,8)
	part[2]=rshift(and(number, mask), 16)
	mask=rshift(mask,8)
	part[3]=rshift(and(number, mask), 8) 
	mask=rshift(mask,8)
	part[4]=and(number, mask)

	ipstr=part[1]
	for (i=2; i<=4; i++) { ipstr=ipstr "." part[i] }

	return ipstr
}


#
# Return an IP v4 addr in binary representation
#
function ip2bin (addr) {
	return num2bin(ip2dec(addr))
}

function bin2dec (binstr,	len) {
	number=0
	p=len=length(binstr)
	for (i=0; i<len; i++) {
		pot=2^i
		digit=substr(binstr,p--,1)
		number=number+(pot*digit)
	}	
	return number
}

#
# Flip bits in a binary string
#
function complement (bitstr,	str,len) {
	p=l=length(bitstr)
	for (i=0; i<l; i++) {
		digit=substr(bitstr,p--,1)
		# Flip digit
		digit=xor(digit,1)
		str=digit str
	}
	return str
}

#########################################################
# CIDR match
# Return 0 if the ip addr matches the CIDR mask
# For IPs outside the network a value of <>0 is returned
#########################################################
function cidrmatch (ipaddr, cidrmask) {
	split(cidrmask,parts,"/")
	cidr_network=parts[1]
	cidr_masklen=parts[2]

	if ( (cidr_masklen < 8) || (cidr_masklen >32) ) return 0

	for (i=1; i<=cidr_masklen; i++) { mask="1" mask }
	while (length(mask) <32) mask=mask "0"

	# Turn to decimals for the sake of handling
	decip=ip2dec(ipaddr)			# Client address
	decmask=bin2dec(mask)			# Netmask
	decnet=ip2dec(cidr_network)		# CIDR network
	deccom=bin2dec(complement(mask))	# mask complement
	fullnet=decnet+deccom

	#
	# Catch Point-2-Point on broadcast addr
	#
	if (decip == fullnet) return 1

	if ( ( decip > decnet) && ( decip < fullnet ) ) {
		matched=1
	} else {matched=0}

	return matched
}
